@@ -1,5 +1,3 @@
-
-
 ### Copyright (C) 2001-2005 Deepayan Sarkar <Deepayan.Sarkar@R-project.org>
 ###
 ### This file is part of the lattice library for R.
@@ -24,7 +22,7 @@
 
 
 prepanel.default.bwplot <-
-    function(x, y, box.ratio,
+    function(x, y, 
              horizontal = TRUE, nlevels,
              origin = NULL, stack = FALSE,
              ...)
@@ -37,10 +35,6 @@ prepanel.default.bwplot <-
 
     if (length(x) && length(y))
     {
-        #if (!is.numeric(x)) x <- as.numeric(x)
-        #if (!is.numeric(y)) y <- as.numeric(y)
-
-        temp <- .5  #* box.ratio/(box.ratio+1)
         if (horizontal)
         {
             if (!is.factor(y)) ## y came from a shingle
@@ -525,14 +519,14 @@ panel.bwplot <-
 
         ## box
 
-        panel.rect(xleft = blist.stats[, 2],
-                   xright = blist.stats[, 4],
-                   y = levels.fos, 
-                   height = blist.height,
-                   lwd = box.rectangle$lwd,
-                   lty = box.rectangle$lty,
-                   fill = fill,
-                   col = box.rectangle$col)
+        lrect(xleft = blist.stats[, 2],
+              xright = blist.stats[, 4],
+              y = levels.fos, 
+              height = blist.height,
+              lwd = box.rectangle$lwd,
+              lty = box.rectangle$lty,
+              col = fill,
+              border = box.rectangle$col)
 
         ## whiskers
 
@@ -604,14 +598,14 @@ panel.bwplot <-
 
         ## box
 
-        panel.rect(ybottom = blist.stats[, 2],
-                   ytop = blist.stats[, 4],
-                   x = levels.fos, 
-                   width = blist.height,
-                   lwd = box.rectangle$lwd,
-                   lty = box.rectangle$lty,
-                   fill = fill,
-                   col = box.rectangle$col)
+        lrect(ybottom = blist.stats[, 2],
+              ytop = blist.stats[, 4],
+              x = levels.fos, 
+              width = blist.height,
+              lwd = box.rectangle$lwd,
+              lty = box.rectangle$lty,
+              col = fill,
+              border = box.rectangle$col)
 
         ## whiskers
 
@@ -982,130 +976,422 @@ panel.violin <-
 
 
 
+### older versions
+
+
+## dotplot.old <-
+##     function(formula,
+##              data = parent.frame(),
+##              panel = "panel.dotplot",
+##              groups = NULL,
+##              ...,
+##              subset = TRUE)
+## {
+##     ## m <- match.call(expand.dots = FALSE)
+##     ## lapply(dots, eval, data, parent.frame())))
+
+##     dots <- list(...)
+##     groups <- eval(substitute(groups), data, parent.frame())
+##     subset <- eval(substitute(subset), data, parent.frame())
+
+##     right.name <- deparse(substitute(formula))
+
+##     try(formula <- eval(formula), silent = TRUE)
+
+##     foo <- substitute(formula)
+
+##     if (!(is.call(foo) && foo[[1]] == "~")) {
+##         formula <- as.formula(paste("~", right.name)) #   deparse(foo)))
+##         environment(formula) <- parent.frame()
+##     }
+
+##     call.list <- c(list(formula = formula, data = data,
+##                         groups = groups,
+##                         subset = subset,
+##                         panel = panel,
+##                         box.ratio = 0),
+##                    dots)
+##     ans <- do.call("bwplot", call.list)
+##     ans$call <- match.call()
+##     ans
+## }
 
 
 
-dotplot <-
-    function(formula,
+## barchart.old <-
+##     function(formula,
+##              data = parent.frame(),
+##              panel = "panel.barchart",
+##              box.ratio = 2,
+##              groups = NULL,
+##              ...,
+##              subset = TRUE)
+## {
+
+##     dots <- list(...)
+##     groups <- eval(substitute(groups), data, parent.frame())
+##     subset <- eval(substitute(subset), data, parent.frame())
+
+##     right.name <- deparse(substitute(formula))
+
+##     try(formula <- eval(formula), silent = TRUE)
+
+##     foo <- substitute(formula)
+
+##     if (!(is.call(foo) && foo[[1]] == "~")) {
+##         formula <- as.formula(paste("~", right.name)) #   deparse(foo)))
+##         environment(formula) <- parent.frame()
+##     }
+
+##     call.list <- c(list(formula = formula, data = data,
+##                         groups = groups,
+##                         subset = subset,
+##                         panel = panel,
+##                         box.ratio = box.ratio),
+##                    dots)
+##     ans <- do.call("bwplot", call.list)
+##     ans$call <- match.call()
+##     ans
+## }
+
+
+## stripplot.old <-
+##     function(formula,
+##              data = parent.frame(),
+##              panel = "panel.stripplot",
+##              jitter = FALSE,
+##              factor = .5,
+##              box.ratio = if (jitter) 1 else 0,
+##              groups = NULL,
+##              ...,
+##              subset = TRUE)
+## {
+##     dots <- list(...)
+##     groups <- eval(substitute(groups), data, parent.frame())
+##     subset <- eval(substitute(subset), data, parent.frame())
+
+##     right.name <- deparse(substitute(formula))
+
+##     try(formula <- eval(formula), silent = TRUE)
+
+##     foo <- substitute(formula)
+
+##     if (!(is.call(foo) && foo[[1]] == "~")) {
+##         formula <- as.formula(paste("~", right.name)) #   deparse(foo)))
+##         environment(formula) <- parent.frame()
+##     }
+##     call.list <- c(list(formula = formula, data = data,
+##                         panel = panel,
+##                         jitter = jitter,
+##                         factor = factor,
+##                         groups = groups,
+##                         subset = subset,
+##                         box.ratio = box.ratio),
+##                    dots)
+##     ans <- do.call("bwplot", call.list)
+##     ans$call <- match.call()
+##     ans
+## }
+
+
+
+### dotplot, barchart and stripplot: essentially wrappers to bwplot
+
+
+
+dotplot <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (is.null(ocall$x)) ocall$x <- formula
+        eval(ocall, parent.frame())
+    }
+    else UseMethod("dotplot")
+}
+
+
+
+## dotplot.numeric <-
+##     function(formula, data = NULL, xlab = deparse(substitute(formula)), ...)
+## {
+##     ## old version:
+##     ## nm <- deparse(substitute(formula))
+##     ## formula <- as.formula(paste("~", nm))
+##     ## or formula <- eval(substitute(~foo, list(foo = substitute(formula))))
+##     ## both have the problem that they don't evaluate the formula
+
+## this last attempt had problems with evaluations
+## (e.g. dotplot(x, groups = a):
+
+##     if (!missing(data))
+##         warning("explicit data specification ignored")
+##     dotplot(~x, data = list(x = formula),
+##             xlab = xlab,
+##             ...)
+## }
+
+dotplot.numeric <-
+    function(x, data = NULL, xlab = deparse(substitute(x)), ...)
+{
+    ocall <- ccall <- match.call()
+    if (!is.null(ccall$data)) 
+        warning("explicit data specification ignored")
+    ccall$data <- list(x = x)
+    ccall$xlab <- xlab
+    ccall$x <- ~x
+    ccall[[1]] <- as.name("dotplot")
+    ans <- eval(ccall, parent.frame())
+    ans$call <- ocall
+    ans
+}
+
+
+
+
+
+
+
+
+
+dotplot.table <-
+    function(x, data = NULL, groups = TRUE, ...)
+{
+    formula <- x
+    ocall <- match.call()
+    ## formula <- eval(substitute(~foo, list(foo = substitute(formula))))
+    data <- as.data.frame(formula)
+    nms <- names(data)
+    freq <- which(nms == "Freq")
+    nms <- nms[-freq]
+    form <- paste(nms[1], "Freq", sep = "~")
+    nms <- nms[-1]
+    len <- length(nms)
+    if (is.logical(groups) && groups && len > 0)
+    {
+        groups <- as.name(nms[len])
+        nms <- nms[-len]
+        len <- length(nms)
+    }
+    else groups <- NULL
+    if (len > 0)
+    {
+        rest <- paste(nms, collapse = "+")
+        form <- paste(form, rest, sep = "|")
+    }
+    ans <-
+        dotplot(as.formula(form), data, groups = eval(groups),
+                 ...)
+    ans$call <- ocall
+    ans
+}
+
+
+dotplot.default <- function(x, ...) dotplot(table(x), ...)
+dotplot.array <- function(x, ...) dotplot(as.table(x), ...)
+dotplot.matrix <- function(x, ...) dotplot(as.table(x), ...)
+
+
+dotplot.formula <-
+    function(x,
              data = parent.frame(),
              panel = "panel.dotplot",
-             groups = NULL,
-             ...,
-             subset = TRUE)
+             ...)
 {
+    ocall <- ccall <- match.call()
+    ccall$data <- data
+    ccall$panel <- panel
+    ccall[[1]] <- as.name("bwplot")
+    ans <- eval(ccall, parent.frame())
+    ans$call <- ocall
+    ans
+}
 
-    ## m <- match.call(expand.dots = FALSE)
-    ## lapply(dots, eval, data, parent.frame())))
 
-    dots <- list(...)
-    groups <- eval(substitute(groups), data, parent.frame())
-    subset <- eval(substitute(subset), data, parent.frame())
-
-    right.name <- deparse(substitute(formula))
-
-    try(formula <- eval(formula), silent = TRUE)
-
-    foo <- substitute(formula)
-
-    if (!(is.call(foo) && foo[[1]] == "~")) {
-        formula <- as.formula(paste("~", right.name)) #   deparse(foo)))
-        environment(formula) <- parent.frame()
+barchart <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (is.null(ocall$x)) ocall$x <- formula
+        eval(ocall, parent.frame())
     }
+    else UseMethod("barchart")
+}
 
-    call.list <- c(list(formula = formula, data = data,
-                        groups = groups,
-                        subset = subset,
-                        panel = panel,
-                        box.ratio = 0),
-                   dots)
-    ans <- do.call("bwplot", call.list)
-    ans$call <- match.call()
+
+barchart.numeric <-
+    function(x, data = NULL, xlab = deparse(substitute(x)), ...)
+{
+    ocall <- ccall <- match.call()
+    if (!is.null(ccall$data)) 
+        warning("explicit data specification ignored")
+    ccall$data <- list(x = x)
+    ccall$xlab <- xlab
+    ccall$x <- ~x
+    ccall[[1]] <- as.name("barchart")
+    ans <- eval(ccall, parent.frame())
+    ans$call <- ocall
     ans
 }
 
 
 
-barchart <-
-    function(formula,
+
+barchart.table <-
+    function(x, data = NULL, groups = TRUE,
+             origin = 0, stack = TRUE, ...)
+{
+    formula <- x
+    ocall <- match.call()
+    if (!is.null(data)) warning("explicit data specification ignored")
+    ## formula <- eval(substitute(~foo, list(foo = substitute(formula))))
+    data <- as.data.frame(formula)
+    nms <- names(data)
+    freq <- which(nms == "Freq")
+    nms <- nms[-freq]
+    form <- paste(nms[1], "Freq", sep = "~")
+    nms <- nms[-1]
+    len <- length(nms)
+    if (is.logical(groups) && groups && len > 0)
+    {
+        groups <- as.name(nms[len])
+        nms <- nms[-len]
+        len <- length(nms)
+    }
+    else groups <- NULL
+    if (len > 0)
+    {
+        rest <- paste(nms, collapse = "+")
+        form <- paste(form, rest, sep = "|")
+    }
+    ans <-
+        barchart(as.formula(form), data,
+                 groups = eval(groups),
+                 ##groups = groups,
+                 origin = origin, stack = stack, 
+                 ...)
+    ans$call <- ocall
+    ans
+}
+
+barchart.default <- function(x, ...) barchart(table(x), ...)
+barchart.array <- function(x, ...) barchart(as.table(x), ...)
+barchart.matrix <- function(x, ...) barchart(as.table(x), ...)
+
+
+barchart.formula <-
+    function(x,
              data = parent.frame(),
              panel = "panel.barchart",
-             box.ratio = 2,
-             groups = NULL,
-             ...,
-             subset = TRUE)
+             box.ratio = 2, 
+             ...)
 {
-
-    dots <- list(...)
-    groups <- eval(substitute(groups), data, parent.frame())
-    subset <- eval(substitute(subset), data, parent.frame())
-
-    right.name <- deparse(substitute(formula))
-
-    try(formula <- eval(formula), silent = TRUE)
-
-    foo <- substitute(formula)
-
-    if (!(is.call(foo) && foo[[1]] == "~")) {
-        formula <- as.formula(paste("~", right.name)) #   deparse(foo)))
-        environment(formula) <- parent.frame()
-    }
-
-    call.list <- c(list(formula = formula, data = data,
-                        groups = groups,
-                        subset = subset,
-                        panel = panel,
-                        box.ratio = box.ratio),
-                   dots)
-    ans <- do.call("bwplot", call.list)
-    ans$call <- match.call()
+    ocall <- ccall <- match.call()
+    ccall$data <- data
+    ccall$panel <- panel
+    ccall$box.ratio <- box.ratio
+    ccall[[1]] <- as.name("bwplot")
+    ans <- eval(ccall, parent.frame())
+    ans$call <- ocall
     ans
 }
 
 
-stripplot <-
-    function(formula,
+stripplot <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (is.null(ocall$x)) ocall$x <- formula
+        eval(ocall, parent.frame())
+    }
+    else UseMethod("stripplot")
+}
+
+
+stripplot.numeric <-
+    function(x, data = NULL, xlab = deparse(substitute(x)), ...)
+{
+    ocall <- ccall <- match.call()
+    if (!is.null(ccall$data)) 
+        warning("explicit data specification ignored")
+    ccall$data <- list(x = x)
+    ccall$xlab <- xlab
+    ccall$x <- ~x
+    ccall[[1]] <- as.name("stripplot")
+    ans <- eval(ccall, parent.frame())
+    ans$call <- ocall
+    ans
+}
+
+
+
+stripplot.formula <-
+    function(x,
              data = parent.frame(),
              panel = "panel.stripplot",
-             jitter = FALSE,
-             factor = .5,
-             box.ratio = if (jitter) 1 else 0,
-             groups = NULL,
-             ...,
-             subset = TRUE)
+             ...)
 {
-
-    dots <- list(...)
-    groups <- eval(substitute(groups), data, parent.frame())
-    subset <- eval(substitute(subset), data, parent.frame())
-
-    right.name <- deparse(substitute(formula))
-
-    try(formula <- eval(formula), silent = TRUE)
-
-    foo <- substitute(formula)
-
-    if (!(is.call(foo) && foo[[1]] == "~")) {
-        formula <- as.formula(paste("~", right.name)) #   deparse(foo)))
-        environment(formula) <- parent.frame()
-    }
-
-    call.list <- c(list(formula = formula, data = data,
-                        panel = panel,
-                        jitter = jitter,
-                        factor = factor,
-                        groups = groups,
-                        subset = subset,
-                        box.ratio = box.ratio),
-                   dots)
-
-    ans <- do.call("bwplot", call.list)
-    ans$call <- match.call()
+    ocall <- ccall <- match.call()
+    ccall$data <- data
+    ccall$panel <- panel
+    ccall[[1]] <- as.name("bwplot")
+    ans <- eval(ccall, parent.frame())
+    ans$call <- ocall
     ans
 }
 
 
-bwplot <-
-    function(formula,
+
+
+### bwplot (the workhorse)
+
+bwplot <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (is.null(ocall$x)) ocall$x <- formula
+        eval(ocall, parent.frame())
+    }
+    else UseMethod("bwplot")
+}
+
+
+
+bwplot.numeric <-
+    function(x, data = NULL, xlab = deparse(substitute(x)), ...)
+{
+    ocall <- ccall <- match.call()
+    if (!is.null(ccall$data)) 
+        warning("explicit data specification ignored")
+    ccall$data <- list(x = x)
+    ccall$xlab <- xlab
+    ccall$x <- ~x
+    ccall[[1]] <- as.name("bwplot")
+    ans <- eval(ccall, parent.frame())
+    ans$call <- ocall
+    ans
+}
+
+
+
+
+bwplot.formula <-
+    function(x,
              data = parent.frame(),
              allow.multiple = is.null(groups) || outer,
              outer = FALSE,
@@ -1130,31 +1416,25 @@ bwplot <-
              subscripts = !is.null(groups),
              subset = TRUE)
 {
-
-    ##m <- match.call(expand.dots = FALSE)
-    ##dots <- m$...
-    ##dots <- lapply(dots, eval, data, parent.frame())
-
+    formula <- x
     dots <- list(...)
-
     groups <- eval(substitute(groups), data, parent.frame())
     subset <- eval(substitute(subset), data, parent.frame())
 
+    ## step 0: hack to get appropriate legend with auto.key = TRUE in
+    ## barchart (default panel only).  The usual default in bwplot is
+    ## appropriate for dotplot and stripplot (groups is usually not
+    ## meaningful in bwplot itself).
+
+    is.standard.barchart <- is.character(panel) && panel == "panel.barchart"
+
     ## Step 1: Evaluate x, y, etc. and do some preprocessing
 
-    formname <- deparse(substitute(formula))
-    formula <- eval(substitute(formula), data, parent.frame())
-
-    if (!inherits(formula, "formula"))
-        formula <- as.formula(paste("~", formname))
-    
     form <-
         latticeParseFormula(formula, data, subset = subset,
                             groups = groups, multiple = allow.multiple,
                             outer = outer, subscripts = TRUE,
                             drop = drop.unused.levels)
-
-
     groups <- form$groups
 
     if (!is.function(panel)) panel <- eval(panel)
@@ -1338,7 +1618,7 @@ bwplot <-
     cond.current.level <- rep(1, number.of.cond)
 
 
-    for (panel.number in seq(length = nplots))
+    for (packet.number in seq(length = nplots))
     {
 
         id <- !id.na
@@ -1358,12 +1638,12 @@ bwplot <-
         {
             if (is.f.y)
             {
-                foo$panel.args[[panel.number]] <-
+                foo$panel.args[[packet.number]] <-
                     list(x = x[id],
                          ##y = as.numeric(y[id]))
                          y = y[id])
                 if (subscripts)
-                    foo$panel.args[[panel.number]]$subscripts <-
+                    foo$panel.args[[packet.number]]$subscripts <-
                         subscr[id]
             }
             else  # shingle
@@ -1378,11 +1658,11 @@ bwplot <-
                     panel.y <- c(panel.y, rep(k,length(tid[tid])))
                     if (subscripts) panel.subscr <- c(panel.subscr, subscr[tid])
                 }
-                foo$panel.args[[panel.number]] <-
+                foo$panel.args[[packet.number]] <-
                     list(x = panel.x,
                          y = panel.y)
                 if (subscripts)
-                    foo$panel.args[[panel.number]]$subscripts <-
+                    foo$panel.args[[packet.number]]$subscripts <-
                         panel.subscr
 
             }
@@ -1391,12 +1671,12 @@ bwplot <-
         {
             if (is.f.x)
             {
-                foo$panel.args[[panel.number]] <-
+                foo$panel.args[[packet.number]] <-
                     ##list(x = as.numeric(x[id]),
                     list(x = x[id],
                          y = y[id])
                 if (subscripts)
-                    foo$panel.args[[panel.number]]$subscripts <-
+                    foo$panel.args[[packet.number]]$subscripts <-
                         subscr[id]
             }
             else   # shingle
@@ -1411,22 +1691,19 @@ bwplot <-
                     panel.x <- c(panel.x, rep(k,length(tid[tid])))
                     if (subscripts) panel.subscr <- c(panel.subscr, subscr[tid])
                 }
-                foo$panel.args[[panel.number]] <-
+                foo$panel.args[[packet.number]] <-
                     list(x = panel.x,
                          y = panel.y)
                 if (subscripts)
-                    foo$panel.args[[panel.number]]$subscripts <-
+                    foo$panel.args[[packet.number]]$subscripts <-
                         panel.subscr
             }
         }
-
         cond.current.level <-
             cupdate(cond.current.level,
                     cond.max.level)
     }
 
-
-
     more.comp <- c(limits.and.aspect(prepanel.default.bwplot,
                                      prepanel = prepanel, 
                                      have.xlim = have.xlim, xlim = xlim, 
@@ -1442,7 +1719,6 @@ bwplot <-
                    cond.orders(foo))
     foo[names(more.comp)] <- more.comp
 
-
     if (is.null(foo$legend) && !is.null(groups) &&
         (is.list(auto.key) || (is.logical(auto.key) && auto.key)))
     {
@@ -1450,8 +1726,8 @@ bwplot <-
             list(list(fun = "drawSimpleKey",
                       args =
                       updateList(list(text = levels(as.factor(groups)),
-                                      points = TRUE,
-                                      rectangles = FALSE,
+                                      points = if (is.standard.barchart) FALSE else TRUE,
+                                      rectangles = if (is.standard.barchart) TRUE else FALSE,
                                       lines = FALSE), 
                                  if (is.list(auto.key)) auto.key else list())))
         foo$legend[[1]]$x <- foo$legend[[1]]$args$x
