@@ -34,7 +34,7 @@ prepanel.default.densityplot <-
              ...)
 {
     if (!is.numeric(x)) x <- as.numeric(x)
-
+    x <- x[!is.na(x)]
     if (length(x) < 1)
         list(xlim = NA,
              ylim = NA,
@@ -52,9 +52,14 @@ prepanel.default.densityplot <-
         if (length(x) > 1)
         {
             h <- do.call("density", c(list(x=x), darg))
+            ## for banking calculations, include only middle 70% values
+            quants <-
+                quantile(x, prob = c(0.15, 0.85),
+                         names = FALSE, na.rm = TRUE)
+            ok <- h$x > quants[1] & h$x < quants[2]
             list(xlim = range(h$x),
                  ylim = range(h$y),
-                 dx = diff(h$x), dy = diff(h$y))
+                 dx = diff(h$x[ok]), dy = diff(h$y[ok]))
         }
         else
             list(xlim = range(x),
@@ -76,8 +81,13 @@ prepanel.default.densityplot <-
                 h <- do.call("density", c(list(x=x[id]), darg))
                 xl <- c(xl, h$x)
                 yl <- c(yl, h$y)
-                dxl <- c(dxl, diff(h$x))
-                dyl <- c(dyl, diff(h$y))
+                ## for banking calculations, include only middle 70% values
+                quants <-
+                    quantile(x[id], prob = c(0.15, 0.85),
+                             names = FALSE, na.rm = TRUE)
+                ok <- h$x > quants[1] & h$x < quants[2]
+                dxl <- c(dxl, diff(h$x[ok]))
+                dyl <- c(dyl, diff(h$y[ok]))
             }
         }
         list(xlim = range(xl, finite = TRUE),
@@ -100,6 +110,7 @@ panel.densityplot <-
              ...)
 {
     x <- as.numeric(x)
+    x <- x[!is.na(x)]
     if (ref)
     {
         reference.line <- trellis.par.get("reference.line")
@@ -136,10 +147,42 @@ panel.densityplot <-
 
 
 
+densityplot <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (is.null(ocall$x)) ocall$x <- formula
+        eval(ocall, parent.frame())
+    }
+    else UseMethod("densityplot")
+}
 
 
-densityplot <-
-    function(formula,
+
+
+densityplot.numeric <-
+    function(x, data = NULL, xlab = deparse(substitute(x)), ...)
+{
+    ocall <- ccall <- match.call()
+    if (!is.null(ccall$data)) 
+        warning("explicit data specification ignored")
+    ccall$data <- list(x = x)
+    ccall$xlab <- xlab
+    ccall$x <- ~x
+    ccall[[1]] <- as.name("densityplot")
+    ans <- eval(ccall, parent.frame())
+    ans$call <- ocall
+    ans
+}
+
+
+
+densityplot.formula <-
+    function(x,
              data = parent.frame(),
              allow.multiple = is.null(groups) || outer,
              outer = !is.null(groups),
@@ -197,14 +240,8 @@ densityplot <-
     groups <- eval(substitute(groups), data, parent.frame())
     subset <- eval(substitute(subset), data, parent.frame())
 
-    formname <- deparse(substitute(formula))
-    formula <- eval(substitute(formula), data, parent.frame())
-
-    if (!inherits(formula, "formula"))
-        formula <- as.formula(paste("~", formname))
-    
     form <-
-        latticeParseFormula(formula, data, subset = subset,
+        latticeParseFormula(x, data, subset = subset,
                             groups = groups, multiple = allow.multiple,
                             outer = outer, subscripts = TRUE,
                             drop = drop.unused.levels)
