@@ -191,14 +191,25 @@ lrect <-
              y = (ybottom + ytop) / 2,
              width = xright - xleft,
              height = ytop - ybottom,
-             just = "center", hjust = NULL, vjust = NULL,
+             col = "transparent",
+             border = "black",
+             lty = 1, lwd = 1, alpha = 1,
+             just = "center",
+             hjust = NULL, vjust = NULL,
              ...)
 {
+    if (is.logical(border))
+    {
+        border <- if (border) col else "transparent"
+    }
     grid.rect(x = x, y = y,
               width = width, height = height,
               default.units = "native",
               just = just, hjust = hjust, vjust = vjust,
-              gp = gpar(...))
+              gp =
+              gpar(fill = col, col = border,
+                   lty = lty, lwd = lwd,
+                   alpha = alpha))
 }
 
 
