@@ -151,11 +151,25 @@ panel.xyplot <-
 
 
 
-xyplot <- function(formula, ...)  UseMethod("xyplot")
+xyplot <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (is.null(ocall$x)) ocall$x <- formula
+        eval(ocall, parent.frame())
+    }
+    else UseMethod("xyplot")
+}
+
+
 
 
 xyplot.formula <-
-    function(formula,
+    function(x,
              data = parent.frame(),
              allow.multiple = is.null(groups) || outer,
              outer = !is.null(groups),
@@ -180,7 +194,6 @@ xyplot.formula <-
              subscripts = !is.null(groups),
              subset = TRUE)
 {
-
     ##dots <- eval(substitute(list(...)), data, parent.frame())
     dots <- list(...)
 
@@ -191,7 +204,7 @@ xyplot.formula <-
 
     ## FIXME: make sure this is done everywhere else
     form <-
-        latticeParseFormula(formula, data, subset = subset,
+        latticeParseFormula(x, data, subset = subset,
                             groups = groups, multiple = allow.multiple,
                             outer = outer, subscripts = TRUE,
                             drop = drop.unused.levels)
@@ -334,7 +347,7 @@ xyplot.formula <-
     cond.current.level <- rep(1, number.of.cond)
 
 
-    for (panel.number in seq(length = nplots))
+    for (packet.number in seq(length = nplots))
     {
 
         id <- !id.na
@@ -347,10 +360,10 @@ xyplot.formula <-
                  & (var <= levels(var)[[cond.current.level[i]]][2]))
             else (as.numeric(var) == cond.current.level[i])
         }
-        foo$panel.args[[panel.number]] <-
+        foo$panel.args[[packet.number]] <-
             list(x = x[id], y = y[id])
         if (subscripts)
-            foo$panel.args[[panel.number]]$subscripts <-
+            foo$panel.args[[packet.number]]$subscripts <-
                 subscr[id]
 
         cond.current.level <-
